package org.bdware.server.irp;

import com.google.gson.reflect.TypeToken;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.sc.util.JsonUtil;
import org.bdware.server.NodeCenterServer;
import org.rocksdb.ReadOptions;
import org.rocksdb.RocksDB;
import org.rocksdb.RocksDBException;
import org.rocksdb.RocksIterator;

import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

public class ViewHandler implements HttpHandler {
    public static final String STATISTICS_KEY = "STATISTICS";
    public static final String COUNT_KEY = "COUNT";
    private static final Logger LOGGER = LogManager.getLogger(ViewHandler.class);
    public static HashMap<String, StatisticsByDay> lhsStatics = new HashMap<>();
    public static HandleRecordCount handleRecordCount;
    RocksDB db;

    public ViewHandler(RocksDB rocksDB) throws RocksDBException {
        db = rocksDB;
        loadFromDB();
        NodeCenterServer.scheduledThreadPool.scheduleWithFixedDelay(
                () -> {
                    try {
                        saveToDB();
                    } catch (RocksDBException e) {
                        LOGGER.error("saving to DB failed! " + e.getMessage());
                    }
                },
                0,
                1,
                TimeUnit.SECONDS);
    }

    public static String getToday() {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
        Date date = new Date(System.currentTimeMillis());
        return formatter.format(date);
    }

    public static void addResolve() {
        String todayDate = ViewHandler.getToday();
        if (lhsStatics.get(todayDate) == null) {
            StatisticsByDay statisticsByDay = new StatisticsByDay();
            statisticsByDay.resolveStatics++;
            lhsStatics.put(todayDate, statisticsByDay);
        } else {
            lhsStatics.get(todayDate).resolveStatics++;
        }
    }

    public static void addRegister() {
        String todayDate = ViewHandler.getToday();
        if (lhsStatics.get(todayDate) == null) {
            StatisticsByDay statisticsByDay = new StatisticsByDay();
            statisticsByDay.registerStatics++;
            lhsStatics.put(todayDate, statisticsByDay);
        } else {
            lhsStatics.get(todayDate).registerStatics++;
        }
    }

    public static void addDelete() {
        String todayDate = ViewHandler.getToday();
        if (lhsStatics.get(todayDate) == null) {
            StatisticsByDay statisticsByDay = new StatisticsByDay();
            statisticsByDay.deleteStatics++;
            lhsStatics.put(todayDate, statisticsByDay);
        } else {
            lhsStatics.get(todayDate).deleteStatics++;
        }
    }

    @Override
    public void handle(HttpExchange httpExchange) throws IOException {
        Map<String, Object> respMap;
        try {
            //获得表单提交数据(post)
            String postString = httpExchange.getRequestURI().getQuery();
            Map<String, String> params = formData2Dic(postString);
            respMap = HandleRequest(params);
            httpExchange.getResponseHeaders().add("Access-Control-Allow-Origin", "*");
            httpExchange.sendResponseHeaders(200, 0);
            OutputStream os = httpExchange.getResponseBody();
            os.write(JsonUtil.toJson(respMap).getBytes());
            os.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private Map<String, Object> HandleRequest(Map<String, String> postInfo) {
        Map<String, Object> respMap = new HashMap<>();
        String action = postInfo.get("action");
        switch (action) {
            case "count":
                respMap = getLHSCount();
                break;
            case "statistics":
                respMap = getLHSStatics(postInfo);
                break;
            default:
                System.out.println("error response");
        }
        return respMap;
    }

    private void loadFromDB() throws RocksDBException {
        byte[] statisticBytes = db.get(STATISTICS_KEY.getBytes());
        if (statisticBytes == null) {
            lhsStatics = new HashMap<>();
        } else {
            lhsStatics = JsonUtil.fromJson(new String(statisticBytes), new TypeToken<HashMap<String, StatisticsByDay>>() {
            }.getType());
        }
        byte[] countBytes = db.get(COUNT_KEY.getBytes());
        if (countBytes == null) {
            handleRecordCount = new HandleRecordCount();
            handleRecordCount.doCount = handleRecordCount.repoCount = handleRecordCount.userCount = 0;
        } else {
            handleRecordCount = JsonUtil.fromJson(new String(countBytes), HandleRecordCount.class);
        }
    }

    private void saveToDB() throws RocksDBException {
        String statisticStr = JsonUtil.toJson(lhsStatics);
        db.put(STATISTICS_KEY.getBytes(), statisticStr.getBytes());
        String countStr = JsonUtil.toJson(handleRecordCount);
        db.put(COUNT_KEY.getBytes(), countStr.getBytes());
    }

    public Map<String, Object> getLHSStatics(Map<String, String> postInfo) {
        if (postInfo.containsKey("day")) {
            return getLHSStaticsWithTime(postInfo);
        }
        HashMap<String, Object> respMap = new HashMap<>();
        for (String key : lhsStatics.keySet()) {
            respMap.put(key + "", JsonUtil.toJson(lhsStatics.get(key)));
        }
        return respMap;
    }

    public Map<String, Object> getLHSStaticsWithTime(Map<String, String> postInfo) {
        long day = Long.parseLong(postInfo.get("day"));
        LOGGER.info(day);
        List<Integer> resolve = new ArrayList<>();
        List<Integer> delete = new ArrayList<>();
        List<Integer> register = new ArrayList<>();

        for (int i = 0; i < day; i++) {
            resolve.add(0);
            delete.add(0);
            register.add(0);
        }

        long interval = 24 * 60 * 60 * 1000;  //一天
        long cur = System.currentTimeMillis();
        long startTime = cur - day * interval;
        // logger.info(JsonUtil.toJson(lhsStatics));
        try {
            for (String k : lhsStatics.keySet()) {
                long time = new SimpleDateFormat("yyyyMMdd").parse(k).getTime();
                if (time < startTime || time > cur) {
                    continue;
                }
                int index = (int) ((time - startTime) / interval);
                //logger.info(index);
                resolve.set(index, resolve.get(index) + lhsStatics.get(k).resolveStatics);
                delete.set(index, delete.get(index) + lhsStatics.get(k).deleteStatics);
                register.set(index, register.get(index) + lhsStatics.get(k).registerStatics);
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }

        HashMap<String, Object> respMap = new HashMap<>();
        respMap.put("resolve", resolve);
        respMap.put("delete", delete);
        respMap.put("register", register);

        return respMap;
    }

    public Map<String, Object> getLHSCount() {
        HashMap<String, Object> respMap = new HashMap<>();
        respMap.put("repoCount", String.valueOf(handleRecordCount.repoCount));
        respMap.put("doCount", String.valueOf(handleRecordCount.doCount));

        ReadOptions readOptions = new ReadOptions().setPrefixSameAsStart(true);
        RocksIterator it1 = db.newIterator(readOptions);

        ArrayList<String> doIDList = new ArrayList<>();
        it1.seek("86.5000.470/do.".getBytes());
        int total = 0;
        while (it1.isValid() && total < 10) {
            doIDList.add(new String(it1.key()));
            total++;
            it1.next();
        }
        it1.close();

        RocksIterator it2 = db.newIterator(readOptions);
        ArrayList<String> repoIDList = new ArrayList<>();
        it2.seek("86.5000.470/doip.".getBytes());
        total = 0;
        while (it2.isValid() && total < 10) {
            repoIDList.add(new String(it2.key()));
            total++;
            it2.next();
        }

        respMap.put("repoIDList", repoIDList);
        respMap.put("doIDList", doIDList);
        return respMap;

    }

    public Map<String, String> formData2Dic(String formData) {
        Map<String, String> result = new HashMap<>();
        if (formData == null || formData.trim().length() == 0) {
            return result;
        }
        final String[] items = formData.split("&");
        Arrays.stream(items).forEach(item -> {
            final String[] keyAndVal = item.split("=");
            if (keyAndVal.length == 2) {
                try {
                    final String key = URLDecoder.decode(keyAndVal[0], "utf8");
                    final String val = URLDecoder.decode(keyAndVal[1], "utf8");
                    result.put(key, val);
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
            }
        });
        return result;
    }

    static class HandleRecordCount {
        int repoCount;
        int doCount;
        int userCount;
    }
}
