package org.bdware.server.nodecenter;

import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.sc.conn.ResultCallback;
import org.bdware.sc.db.KeyValueDBUtil;
import org.bdware.sc.db.MultiIndexTimeDBUtilIntf;
import org.bdware.sc.util.JsonUtil;
import org.bdware.server.NodeCenterServer;
import org.bdware.server.action.Action;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

// CenterPortal的所有日志和统计数据
public class LogActions {
    private static final Logger LOGGER = LogManager.getLogger(LogActions.class);

    NCManagerAction managerAction;

    public LogActions(NCManagerAction managerAction) {
        this.managerAction = managerAction;
    }

    private void simpleReply(ResultCallback resultCallback, String action, Object data) {
        Map<String, Object> jsonResult = new HashMap<>();
        jsonResult.put("action", action);
        jsonResult.put("data", data);
        resultCallback.onResult(jsonResult);
    }

    private void queryInternal(
            String actResp,
            MultiIndexTimeDBUtilIntf db,
            JsonObject json,
            ResultCallback resultCallback) {
        long start = System.currentTimeMillis();
        if (!json.has("start")) {
            simpleReply(resultCallback, actResp, new ArrayList<>());
            return;
        }
        long startTime = json.get("start").getAsLong();
        long endTime = System.currentTimeMillis();
        if (json.has("end")) {
            endTime = json.get("end").getAsLong();
        }
        String[] category = new String[]{null};
        if (json.has("category")) {
            category = json.get("category").getAsString().split(",");
        }
        Map<String, Object> data = new HashMap<>();
        for (String str : category) {
            List<JsonObject> array = db.queryByDateAsJson(str, startTime, endTime);
            if (str == null) data.put("primary", array);
            else data.put(str, array);
        }
        simpleReply(resultCallback, actResp, data);
        long end = System.currentTimeMillis();
        LOGGER.debug("[queryInternal:time]" + (end - start));
    }

    private void countLogByCategoryInternal(
            String actResp,
            MultiIndexTimeDBUtilIntf db,
            JsonObject json,
            ResultCallback resultCallback) {
        if (!json.has("start")) {
            simpleReply(resultCallback, actResp, new ArrayList<>());
            return;
        }
        long startTime = json.get("start").getAsLong();
        long endTime = System.currentTimeMillis();
        if (json.has("end")) {
            endTime = json.get("end").getAsLong();
        }
        long interval = json.get("interval").getAsLong();
        String[] category = new String[]{null};
        if (json.has("category")) {
            category = json.get("category").getAsString().split(",");
        }
        JsonObject data = new JsonObject();
        for (String str : category) {
            JsonArray array = db.countInInterval(str, startTime, interval, endTime);
            if (str == null) data.add("primary", array);
            else data.add(str, array);
        }
        simpleReply(resultCallback, actResp, data);
    }

    @Action(userPermission = 1 << 8, async = true)
    public void queryActionLog(JsonObject json, ResultCallback resultCallback) {
        queryInternal("onQueryActionLog", NodeCenterServer.nodeHttpLogDB, json, resultCallback);
    }

    @Action(userPermission = 1 << 8, async = true)
    public void countActionLogByCategory(JsonObject json, ResultCallback resultCallback) {
        countLogByCategoryInternal(
                "onCountActionLogByCategory", NodeCenterServer.nodeHttpLogDB, json, resultCallback);
    }

    @Action(userPermission = 1 << 8, async = true)
    public void queryCMLog(JsonObject json, ResultCallback resultCallback) {
        queryInternal("onQueryCMLog", NodeCenterServer.nodeTcpLogDB, json, resultCallback);
    }

    @Action(userPermission = 1 << 8, async = true)
    public void countCMLogByCategory(JsonObject json, ResultCallback resultCallback) {
        countLogByCategoryInternal(
                "onCountCMLogByCategory", NodeCenterServer.nodeTcpLogDB, json, resultCallback);
    }

    @Action(userPermission = 1 << 9)
    public void queryUserStat(JsonObject json, ResultCallback resultCallback) {
        long start = System.currentTimeMillis();
        long userList = KeyValueDBUtil.instance.getCount(NCTables.NodeUser.toString());
        long applylist = KeyValueDBUtil.instance.getCount(NCTables.ApplyRole.toString());
        JsonObject jsonResult = new JsonObject();
        jsonResult.addProperty("action", "onQueryUserStat");
        jsonResult.addProperty("userListCount", userList);
        jsonResult.addProperty("applyListCount", applylist);
        resultCallback.onResult(jsonResult);
        long end = System.currentTimeMillis();
        LOGGER.debug("[queryUserStat:time]" + (end - start));
    }

    @Action(userPermission = 1 << 9, async = true)
    public void listNodes(JsonObject json, ResultCallback resultCallback) {
        long start = System.currentTimeMillis();
        final String pubKey = managerAction.pubKey;
        LOGGER.debug("[listNodes] managerAction.pubKey " + pubKey);
        Map<String, CMNode> nodeinfos = NodeCenterActions.nodeInfos; // 所有在线节点?
        final Map<String, CMNode> cinodeinfos = new HashMap<>();
        List<String> dbnodes = KeyValueDBUtil.instance.getKeys(NCTables.NodesDB.toString());
        List<CMNode> onlineNodes = new ArrayList<>();

        Map<String, Object> ret = new HashMap<>();
        if (KeyValueDBUtil.instance
                .getValue(NCTables.ConfigDB.toString(), "__CenterManager__")
                .contains(pubKey)) {
            LOGGER.debug("is center manager");
            LOGGER.debug("dbnodes " + dbnodes.toString());
            for (CMNode node : nodeinfos.values()) {
                LOGGER.debug("Offline node " + node.nodeName + node.pubKey);
                if (dbnodes.contains(node.pubKey)) {
                    dbnodes.remove(node.pubKey);
                    LOGGER.debug("Delete Online node " + node.pubKey);
                    onlineNodes.add(node);
                }
            }
            ret.put("action", "onListNodes");
            ret.put("online", onlineNodes);
            ret.put("offline", dbnodes);
            resultCallback.onResult(ret);
            return;
        }
        for (Map.Entry<String, CMNode> entry : nodeinfos.entrySet()) {
            LOGGER.debug("Key = " + entry.getKey() + ", Value = " + entry.getValue());
            String cimanager = entry.getValue().cimanager;
            if (!cimanager.contains(pubKey)) {
                cinodeinfos.put(entry.getKey(), entry.getValue());
            }
        }
        ret.put("action", "onListNodes");
        ret.put("online", cinodeinfos);
        ret.put("offline", new ArrayList<String>());
        // 合约管理员看不到offline nodes
        resultCallback.onResult(ret);
        long end = System.currentTimeMillis();
        LOGGER.debug("[listNodes:time]" + (end - start));
    }

}
